/// <reference types="isaac-typescript-definitions" />
import { Feature } from "../../private/Feature";
/**
 * This feature provides a way for end-users to get the `EntityPlayer` object for the other Tainted
 * Lazarus.
 */
export declare class TaintedLazarusPlayers extends Feature {
    vConditionalFunc: () => boolean;
    private readonly postPlayerInit;
    /**
     * Indexes are the `PtrHash`, values are the `EntityPtr` of the *other* Lazarus.
     *
     * When starting a run, the `POST_PLAYER_INIT` callback will fire first for Dead Tainted Lazarus,
     * then for Tainted Lazarus. When continuing a run, the `POST_PLAYER_INIT` callback will fire
     * first for the character that is currently active. Thus, since the order of the characters is
     * not certain, we insert each of their pointers into a queue, and then only populate the map when
     * we have one Tainted Lazarus and one Dead Tainted Lazarus.
     */
    private checkDequeue;
    /**
     * Helper function to get the other version of Tainted Lazarus.
     *
     * - On Tainted Lazarus, returns the player object for Dead Tainted Lazarus.
     * - On Dead Tainted Lazarus, returns the player object for Tainted Lazarus.
     * - Returns undefined if player object retrieval failed for any reason.
     *
     * If you call the `EntityPlayer.Exists` method on the returned object, it will return false.
     * However, you can still call the other methods like you normally would (e.g.
     * `EntityPlayer.AddCollectible`).
     *
     * In order to use this function, you must upgrade your mod with
     * `ISCFeature.CHARACTER_HEALTH_CONVERSION`.
     *
     * @public
     */
    getTaintedLazarusSubPlayer(player: EntityPlayer): EntityPlayer | undefined;
}
//# sourceMappingURL=TaintedLazarusPlayers.d.ts.map